#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright @2024 AI. Inspur Inc.
#
# @author: suojiayi <suojiayi@inspur.com>
# @date: 2025/05/13

import json
from collections import defaultdict
import pandas as pd

def classify_data_by_ins_tag(input_file, output_jsonl, output_excel):
    """
    根据 ins_tag_label 中的标签对数据进行分类。
    每个类型对应一个列表，存储去掉了 embedding 的数据。
    输出 JSONL 文件按类型的数据量从多到少排序，并记录每种类型的数量。
    同时将类型及其数量导出到 Excel 表格中。
    """

    # 初始化分类字典
    classified_data = defaultdict(list)

    # 读取输入文件
    with open(input_file, 'r', encoding='utf-8') as f:
        for line in f:
            record = json.loads(line.strip())

            # 提取 ins_tag_label 部分
            ins_tag_label = record.get("ins_tag_label", "[]")
            try:
                ins_tag_label = json.loads(ins_tag_label)
            except json.JSONDecodeError:
                #print(f"JSON decode error: {ins_tag_label}")
                ins_tag_label = []

            # 如果 ins_tag_label 存在，则根据类型分类
            if ins_tag_label:
                for tag in ins_tag_label:
                    record_without_embedding = {k: v for k, v in record.items() if k != "embedding"}
                    classified_data[tag].append(record_without_embedding)
            else:
                # 如果没有 ins_tag_label，则归类到 "null"
                record_without_embedding = {k: v for k, v in record.items() if k != "embedding"}
                classified_data["null"].append(record_without_embedding)

    # 对每个 data_list 按照 score 降序排序（如果不存在 score，默认为 0）
    for tag in classified_data:
        classified_data[tag].sort(key=lambda x: x.get('score', 0), reverse=True)

    # 过滤掉长度小于83的类别，并排除 "null" 类别
    filtered_data = {
        tag: records for tag, records in classified_data.items()
        if len(records) >= 83 and tag != "null"
    }

    # 按照记录数从多到少排序
    sorted_filtered_data = sorted(filtered_data.items(), key=lambda x: len(x[1]), reverse=True)

    # 写入 JSONL 文件
    total_types = len(sorted_filtered_data)
    with open(output_jsonl, 'w', encoding='utf-8') as out_f:
        for type_name, data_list in sorted_filtered_data:
            entry = {
                type_name: data_list,
                #"count": len(data_list)
            }
            out_f.write(json.dumps(entry, ensure_ascii=False) + '\n')

    # 准备 Excel 数据
    excel_data = []
    for type_name, data_list in sorted_filtered_data:
        excel_data.append({"Type": type_name, "Count": len(data_list)})

    # 导出到 Excel 文件
    df = pd.DataFrame(excel_data)
    df.to_excel(output_excel, index=False)

    print(f"Total types after filtering: {total_types}")
    return total_types


# 示例用法
input_file = './dhbq/dhbq_merged_with_score.jsonl'
output_jsonl = './dhbq/dhbq_count_instag.jsonl'
output_excel = './dhbq/dhbq_count_instag.xlsx'

total_types = classify_data_by_ins_tag(input_file, output_jsonl, output_excel)
print(f"Final types found: {total_types}")