#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright @2024 AI. Inspur Inc.
#
# @author: suojiayi <suojiayi@inspur.com>
# @date: 2025/05/13

import json
import requests
import pandas as pd
from concurrent.futures import ThreadPoolExecutor, as_completed

def read_jsonl_lines_in_batches(file_path, batch_size=10000):
    """按批次读取 JSONL 文件"""
    batch = []
    with open(file_path, mode="r", encoding="utf-8") as f:
        for line in f:
            try:
                batch.append(json.loads(line.strip()))
                if len(batch) == batch_size:
                    yield batch
                    batch = []
            except json.JSONDecodeError as e:
                print(f"Error decoding JSON: {e}")
        if batch:
            yield batch


def process_data_concurrently(data_list, api_url, headers, max_workers=10):
    """并发处理数据并调用 API"""
    result_data = []

    def process_single_data(data):
        try:
            query = data.get('data')
            if query:
                question = query[-1]['content']
                conversation = query[:len(query)-1]
                input_content = f'''任务说明：
请对给定的历史多轮对话进行分析，结合上下文逻辑和多轮对话特性，识别最后一轮问题的缺失信息， 对改写后的问题进行评分（1-5分）。

改写要求：
忠实性：确保改写后的问题与原问题意图一致，无歧义、无信息偏差，且无冗余修改。
完整性：若上下文可提供必要信息，应填补空白，使问题更完整，同时保持语义连贯。
保守性：若上下文无法有效澄清原问题，则保持原问题不变（不强行补全）。
语义一致性：不得改变原问题的语义或缩小其范围。
中立性：不得针对模型回答进行优化（即避免引导性改写）。
时间相关性：除非原问题涉及时间，否则不得引入无关时间信息。

评分标准（1-5分）：
分数	        标准描述
5（优秀）	完全符合原意图，填补信息精准且必要，语义连贯，无冗余或无关修改。
4（良好）	基本符合原意图，补充信息合理但可能稍显冗余或不够精准，语义基本连贯。
3（中等）	大体保留原意图，但补充信息可能不精准或必要性不足，语义稍显生硬。
2（较差）	部分偏离原意图，补充信息冗余或关联性弱，语义不连贯或存在歧义。
1（差）	    严重偏离原意图，补充信息错误或无关，语义混乱，或强行补全导致问题失真。
特殊情况处理：
上下文无关：若无法从上下文合理推断缺失信息，则保持原问题不变（5分）。
强行补全：若补充信息导致语义改变或范围收窄，应扣分（视情况给2分或更低）。

原多轮对话及问题：{conversation}
改写后问题：{question}
评分：请基于上述标准给出1-5分的评分。
输出格式：
仅返回1-5之间的整数分数，无需解释。'''
                # print(input_content)
                response = requests.post(
                    api_url,
                    headers=headers,
                    json={
                        "model": "Qwen2.5-72B-Instruct",
                        "stream": False,
                        "temperature": 0.01,
                        "messages": [{"role": "user", "content": input_content}]
                    }
                )
                if response.status_code == 200:
                    try:
                        content = response.json()["choices"][0]["message"]["content"]
                    except (KeyError, IndexError, json.JSONDecodeError):
                        content = "无法解析返回内容"
                else:
                    content = f"API请求失败，状态码：{response.status_code}"
                return {
                    "uid": data.get('uid'),
                    "data": query,
                    "answer": content
                }
        except Exception as e:
            print(e)
            return None

    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        futures = [executor.submit(process_single_data, data) for data in data_list]
        for future in as_completed(futures):
            result = future.result()
            if result:
                result_data.append(result)

    return result_data


def save_to_excel_in_batches(data_list, output_file, batch_size=10000):
    """按批次保存数据到 Excel 文件"""
    df = pd.DataFrame(data_list)
    writer = pd.ExcelWriter(output_file, engine='openpyxl')
    for i in range(0, len(df), batch_size):
        batch_df = df.iloc[i:i + batch_size]
        batch_df.to_excel(writer, index=False, startrow=i)
    writer.close()
    print(f"数据已成功保存到 {output_file}")

def save_to_jsonl_in_batches(data_list, output_file, batch_size=10000):
    """按批次保存数据到 JSONL 文件"""
    with open(output_file, 'w', encoding='utf-8') as f:
        for i in range(0, len(data_list), batch_size):
            # 获取当前批次的数据
            batch_data = data_list[i:i + batch_size]
            # 将每个数据对象写入文件，每行一个 JSON 对象
            for item in batch_data:
                f.write(json.dumps(item, ensure_ascii=False) + '\n')
    print(f"数据已成功保存到 {output_file}")

if __name__ == "__main__":
    #output_excel_file = 'result-taoli-5.xlsx'
    api_url = "http://100.105.48.35:8000/v1/chat/completions"
    # api_url = "http://100.105.107.150:8000/v1/chat/completions"
    headers = {
        "Content-Type": "application/json",
        "Authorization": "7c3eafb5-2d6e-100d-ab0f-7b2c1cdafb3c"
    }

    #file_path = '/dataset-pvc/suojiayi/new/train_prepare/20250423_020157/tmp_data/instruct_data_BELLE_Multiturn_Chat_filtered_2504232014.jsonl'
    file_path = '/data/suojiayi/buquan/split_dhbq/part_1.jsonl'
    output_file = './dhbq/duihuabuquan_score_1.jsonl'
    #file_path = '/dataset-pvc/suojiayi/new/train_prepare/20250423_020157/tmp_data/instruct_data_COIG_filtered_2504212014.jsonl'
    all_results = []
    for batch in read_jsonl_lines_in_batches(file_path, batch_size=10000):
        processed_batch = process_data_concurrently(batch, api_url, headers, max_workers=20)
        all_results.extend(processed_batch)
        # save_to_excel_in_batches(all_results, output_excel_file, batch_size=23000)
        save_to_jsonl_in_batches(all_results, output_file, batch_size=10000)
