#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright @2024 AI. Inspur Inc.
#
# @author: suojiayi <suojiayi@inspur.com>
# @date: 2025/05/13

import json
import pandas as pd
import torch
import torch.nn.functional as F

def load_excel_column(file_path, column_index=2):
    """读取 Excel 文件的第三列，并将每一行解析为 dict"""
    df = pd.read_excel(file_path)

    result = []
    seen_uids = set()

    for _, row in df.iterrows():
        try:
            data = json.loads(row.iloc[column_index])
            if isinstance(data, dict) and 'uid' in data:
                if data['uid'] not in seen_uids:
                    seen_uids.add(data['uid'])
                    result.append(data)
        except (json.JSONDecodeError, IndexError):
            continue
    return result


def process_jsonl_file(file_path, top_n_per_group, result, seen_uids):
    """
    读取 jsonl 文件，对每一行 group 数据，尽力从中选出 top_n_per_group 条不重复数据
    如果无法选够指定数量的数据，则跳过该 group 并打印警告信息。
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        for line_idx, line in enumerate(f):
            line = line.strip()
            if not line:
                continue
            try:
                entry = json.loads(line)

                # 获取 group key 和对应的数据列表
                key = next(k for k in entry if k != "count")
                records = entry[key]

                selected = []
                added_count = 0

                # 遍历 records，直到选够 top_n_per_group 个不重复的数据
                for item in records:
                    uid = item.get('uid')

                    if added_count >= top_n_per_group:
                        break  # 已经选够了，退出

                    if uid and uid not in seen_uids:
                        seen_uids.add(uid)
                        selected.append(item)
                        result.append(item)
                        added_count += 1

                # 如果最终无法选满 top_n_per_group 条，可以给出警告或处理
                if added_count < top_n_per_group:
                    print(f"[Group {key}] Only found {added_count}/{top_n_per_group} unique items. Skipping this group.")
                    # 如果需要的话，可以从结果中移除已添加的记录
                    for item in selected:
                        result.remove(item)
                        seen_uids.discard(item.get('uid'))
                else:
                    print(f"[Group {key}] Successfully added {added_count} items.")

            except Exception as e:
                print(f"Error processing line {line_idx + 1}: {e}")
                continue

def is_similar_to_existing(embedding, existing_embeddings, threshold=0.85):
    """
    判断当前 embedding 是否与已有 embeddings 的最大余弦相似度超过阈值
    """
    if not existing_embeddings:
        return False

    current_emb = torch.tensor(embedding, dtype=torch.float32)
    existing_emb = torch.tensor(existing_embeddings, dtype=torch.float32)

    similarity = F.cosine_similarity(current_emb.unsqueeze(0), existing_emb)
    return (similarity > threshold).any().item()

def process_cluster_jsonl(file_path, top_n_per_group, result, seen_uids, similarity_threshold=0.5):
    """
    处理 dhbq_count_cluster.jsonl，每组最多取 top_n_per_group 条，
    并根据 embedding 字段进行余弦相似度过滤。
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        for line_idx, line in enumerate(f):
            line = line.strip()
            if not line:
                continue
            try:
                entry = json.loads(line)
                key = next(k for k in entry if k != "count")
                records = entry[key]

                selected = []
                added_count = 0
                existing_embeddings = [item['embedding'] for item in result if 'embedding' in item]  # 已有所有 embedding

                for item in records:
                    uid = item.get('uid')
                    emb = item.get('embedding')

                    if added_count >= top_n_per_group:
                        break

                    if not emb or not isinstance(emb, list):
                        print(f"Skipping item {uid}: missing or invalid embedding.")
                        continue

                    if uid and uid not in seen_uids:
                        if is_similar_to_existing(emb, existing_embeddings, similarity_threshold):
                            print(f"Skipping item {uid} due to high similarity.")
                            continue

                        seen_uids.add(uid)
                        selected.append(item)
                        result.append(item)
                        existing_embeddings.append(emb)
                        added_count += 1

                print(f"[Group {key}] Successfully added {added_count} items.")

            except Exception as e:
                print(f"Error processing line {line_idx + 1}: {e}")
                continue

def main():
    result = []
    seen_uids = set()

    # Step 1: 读取 Excel 第三列
    print("Step 1: Reading Excel file...")
    excel_data = load_excel_column("./dhbq/frequency-score-5.xlsx", column_index=2)
    for item in excel_data:
        uid = item.get('uid')
        if uid and uid not in seen_uids:
            seen_uids.add(uid)
            result.append(item)

    # Step 2: 处理 dhbq_count_prompt_label.jsonl，每组前3条
    print("Step 2: Processing dhbq_count_prompt_label.jsonl...")
    process_jsonl_file("./dhbq/dhbq_count_prompt_label.jsonl", 170, result, seen_uids)

    # Step 3: 处理 dhbq_count_instag.jsonl，每组前130条
    print("Step 3: Processing dhbq_count_instag.jsonl...")
    process_jsonl_file("./dhbq/dhbq_count_instag.jsonl", 3, result, seen_uids)

    # # Step 4: 处理 dhbq_count_cluster.jsonl，每组前4条
    # print("Step 4: Processing dhbq_count_cluster.jsonl...")
    # process_jsonl_file("./dhbq/dhbq_count_cluster_0513.jsonl", 5, result, seen_uids)
    print("Step 4: Processing dhbq_count_cluster.jsonl...")
    process_cluster_jsonl("./dhbq/dhbq_count_cluster_0513.jsonl", 5, result, seen_uids, similarity_threshold=0.5)

    # 最终写入输出文件
    output_file = "./dhbq/merged_result_for_more_test_0513.jsonl"
    with open(output_file, 'w', encoding='utf-8') as f:
        for item in result:
            f.write(json.dumps(item, ensure_ascii=False) + '\n')

    print(f"Total merged items: {len(result)}")
    print(f"Saved to: {output_file}")


if __name__ == "__main__":
    main()


